/**********
**          Driver.cpp
*/

extern "C"
{
    #include <ntddk.h>
}

#include "s2e-win32.h"
#include "..\Common\IoCtl.h"


long testVar;
VOID TestFunc(){
	if (testVar == 0){
		DbgPrint("testVar = 0\n");
	}
	else{
		DbgPrint("ELSE CASE: testVar != 0\n");
	}
}


/****
***     OnCreateCleanupClose
**
*/
NTSTATUS
OnCreateCleanupClose(__in PDEVICE_OBJECT    pDeviceObj,
                     __in PIRP              pIrp)
{
    pIrp->IoStatus.Status = STATUS_SUCCESS;
    pIrp->IoStatus.Information = 0;
    IoCompleteRequest(pIrp, IO_NO_INCREMENT);

    return STATUS_SUCCESS;
}


/****
***     OnDeviceIoControl
**
*/
NTSTATUS
OnDeviceIoControl(__in PDEVICE_OBJECT   pDeviceObj,
                  __in PIRP             pIrp)
{
    PIO_STACK_LOCATION			pStack;
    NTSTATUS					nts;
    PKERNEL_MATH_REQUEST		pRequest;
    PKERNEL_MATH_RESPONSE		pResponse;
	PKERNEL_SYMBOLIC_REQUEST	SymbolicRequest;
	PKERNEL_SYMBOLIC_RESPONSE	SymbolicResponse;

    pStack = IoGetCurrentIrpStackLocation(pIrp);


    switch(pStack->Parameters.DeviceIoControl.IoControlCode)
    {
    case IOCTL_ADD_THESE_TWO_NUMBERS:
    case IOCTL_SUBTRACT_THESE_TWO_NUMBERS:

        if (pStack->Parameters.DeviceIoControl.InputBufferLength !=
            sizeof(KERNEL_MATH_REQUEST) ||
            pStack->Parameters.DeviceIoControl.OutputBufferLength !=
            sizeof(KERNEL_MATH_RESPONSE))
        {
            nts = STATUS_INVALID_BUFFER_SIZE;
            pIrp->IoStatus.Information = 0;
            break;
        }

        
        ASSERT(pIrp->AssociatedIrp.SystemBuffer != NULL);

        pRequest = (PKERNEL_MATH_REQUEST)pIrp->AssociatedIrp.SystemBuffer;
        pResponse = (PKERNEL_MATH_RESPONSE)pIrp->AssociatedIrp.SystemBuffer;


        if (pStack->Parameters.DeviceIoControl.IoControlCode == IOCTL_ADD_THESE_TWO_NUMBERS)
            pResponse->r = pRequest->x + pRequest->y;
        else
            pResponse->r = pRequest->x - pRequest->y;

        nts = STATUS_SUCCESS;

        
        pIrp->IoStatus.Information = sizeof(KERNEL_MATH_RESPONSE);

		s2e_make_symbolic(&testVar, 4, "Sym_TestVar");

		s2e_enable_forking();
		TestFunc();
		s2e_disable_forking();

		
        break;

	case IOCTL_INSERT_SYMBOLIC_DATA:
		if (pStack->Parameters.DeviceIoControl.InputBufferLength !=
            sizeof(KERNEL_SYMBOLIC_REQUEST) ||
            pStack->Parameters.DeviceIoControl.OutputBufferLength !=
            sizeof(KERNEL_SYMBOLIC_RESPONSE))
        {
            nts = STATUS_INVALID_BUFFER_SIZE;
            pIrp->IoStatus.Information = 0;
            break;
        }

		ASSERT(pIrp->AssociatedIrp.SystemBuffer != NULL);

        SymbolicRequest = (PKERNEL_SYMBOLIC_REQUEST)pIrp->AssociatedIrp.SystemBuffer;
        SymbolicResponse = (PKERNEL_SYMBOLIC_RESPONSE)pIrp->AssociatedIrp.SystemBuffer;

		//s2e_make_symbolic(SymbolicRequest->TargetAddress, SymbolicRequest->Size, SymbolicRequest->SymbolicName);
		s2e_make_symbolic(&testVar, 4, "Sym_TestVar");

		s2e_enable_forking();
		TestFunc();
		s2e_disable_forking();

		break;

    default:
        
        nts = STATUS_INVALID_DEVICE_REQUEST;
        pIrp->IoStatus.Information = 0;
    }

    
    pIrp->IoStatus.Status = nts;
    IoCompleteRequest(pIrp, IO_NO_INCREMENT);

    return nts;
}


/****
***     OnDriverUnload
*/
VOID OnDriverUnload(__in PDRIVER_OBJECT  pDriverObj)
{
    UNICODE_STRING  usSymbolicLink = RTL_CONSTANT_STRING(L"\\DosDevices\\SymbolicKernel");

    
    ASSERT(pDriverObj->DeviceObject != NULL);

    IoDeleteSymbolicLink(&usSymbolicLink);
    IoDeleteDevice(pDriverObj->DeviceObject);
}


/****
***     DriverEntry
**
*/
extern "C" NTSTATUS
DriverEntry(__in PDRIVER_OBJECT     pDriverObj,
            __in PUNICODE_STRING    pusRegistryPath)
{
    UNICODE_STRING  usDeviceName = RTL_CONSTANT_STRING(L"\\Device\\SymbolicKernel");
    UNICODE_STRING  usSymbolicLink = RTL_CONSTANT_STRING(L"\\DosDevices\\SymbolicKernel");
    NTSTATUS        nts;
    PDEVICE_OBJECT  pDeviceObj;

	
	DbgPrint("Address of testVar = %p\n", &testVar);
    
    pDriverObj->DriverUnload = OnDriverUnload;

    pDriverObj->MajorFunction[IRP_MJ_CREATE] =
    pDriverObj->MajorFunction[IRP_MJ_CLEANUP] =
    pDriverObj->MajorFunction[IRP_MJ_CLOSE] = OnCreateCleanupClose;
    pDriverObj->MajorFunction[IRP_MJ_DEVICE_CONTROL] = OnDeviceIoControl;

    nts = IoCreateDevice(pDriverObj,
                         0,
                         &usDeviceName,
                         FILE_DEVICE_UNKNOWN,
                         0,
                         FALSE,
                         &pDeviceObj);

    
    if (!NT_SUCCESS(nts))
    {
        ASSERT(FALSE);
        return nts;
    }

    
    nts = IoCreateSymbolicLink(&usSymbolicLink,
                               &usDeviceName);

    
    if (!NT_SUCCESS(nts))
    {
        IoDeleteDevice(pDeviceObj);

        ASSERT(FALSE);
        return nts;
    }

    return STATUS_SUCCESS;
}

